import * as React from 'react';
import { noop } from 'lodash';
import classNames from 'classnames';

// import {useHover} from '@react-aria/interactions';

import Content from './content';
// import Animate from './animate';
// import 'src/styles/scroolbar.css';

// @ts-ignore
import gs from '@ladifire-css/sass/index.scss';
import ss from '@ladifire-css/sass/components/scrollbar.scss';

const E = function e( r: any ) {
  return r.stopPropagation();
};

export interface ScrollbarProps {
  children?: any;
  width: number;
  height: number;
  contentHeight: number;
  color?: string;
  className?: string;
  monkey?: boolean;
  fade?: boolean;
  anchor: string; // 'top' | 'bottom';
  role: string;
  ariaLabel?: string;
  onScroll: (event: any, r?: any) => void;
  onContentScroll?: (event: any) => void;
  initialScrollTop?: number;
  onTrackClick: any;
  onKeyDown: any;
  onMouseDown?: any;
  ref?: any;
  isEmpty?: boolean;
}

export interface ScrollbarState {
  top: number;
  height: number;
  hidden: boolean;
}

export class Scrollbar extends React.PureComponent<ScrollbarProps, ScrollbarState> {
  static displayName = 'Scrollbar';
  // static propTypes = {
  //   children: PropTypes.node,
  //   width: PropTypes.number.isRequired,
  //   height: PropTypes.number.isRequired,
  //   contentHeight: PropTypes.number,
  //   color: PropTypes.string,
  //   className: PropTypes.string,
  //   monkey: PropTypes.bool,
  //   fade: PropTypes.bool,
  //   anchor: PropTypes.oneOf( [ 'top', 'bottom' ] ),
  //   role: PropTypes.string,
  //   ariaLabel: PropTypes.string,
  //   onScroll: PropTypes.func,
  //   onContentScroll: PropTypes.func,
  //   onTrackClick: PropTypes.func,
  //   initialScrollTop: PropTypes.number,
  // };

  static defaultProps = {
    children: undefined,
    style: undefined,
    contentHeight: undefined,
    color: undefined,
    className: undefined,
    monkey: false,
    fade: false,
    anchor: 'top',
    role: 'presentation',
    ariaLabel: undefined,
    onScroll: undefined,
    onContentScroll: noop,
    onTrackClick: undefined,
    initialScrollTop: 0,
  };

  private scroller: any;
  private currentScrollTop: number;
  public dragging: boolean;
  private delta: { top?: number; y?: number; speed?: number };
  private trackInterval: any;
  private trackTimeout: any;
  private trackAnimation: any;
  public track: any;

  constructor( props: ScrollbarProps ) {
    super( props );

    this.scroller = null;
    this.currentScrollTop = props.initialScrollTop || 0;
    this.dragging = false;
    this.delta = {
      y: undefined,
      top: undefined,
      speed: undefined,
    };
    this.trackInterval = void 0;
    this.trackTimeout = void 0;
    this.trackAnimation = void 0;
    this.onScroll = this.onScroll.bind( this );
    this.onDragStart = this.onDragStart.bind( this );
    this.onDragEnd = this.onDragEnd.bind( this );
    this.onDrag = this.onDrag.bind( this );
    this.onTrackDown = this.onTrackDown.bind( this );
    this.onTrackUp = this.onTrackUp.bind( this );
    this.setRef = this.setRef.bind( this );
    this.setTrackRef = this.setTrackRef.bind( this );
    this.state = {
      height: 0,
      top: 0,
      hidden: props.fade,
    };
  }

  componentDidMount() {
    this.props.initialScrollTop && ( this.scroller.scrollTop = this.props.initialScrollTop );
    this.update();
  }

  componentDidUpdate() {
    this.update();
  }

  componentWillUnmount() {
    this.trackAnimation && this.trackAnimation.cancel();
    clearInterval( this.trackInterval );
    clearTimeout( this.trackTimeout );
    window.removeEventListener( 'mouseup', this.onTrackUp );
  }

  public static track = {
    to: function e( r: any ) {
      const a = r.y,
        n = r.scrollTop,
        t = r.scrollHeight,
        s = r.trackHeight,
        l = r.barTop,
        o = r.barHeight;
      const i = a - l - o / 2;
      return n + i * ( t / s );
    },
    page: function e( r: any ) {
      const a = r.y,
        n = r.scrollTop,
        t = r.trackHeight,
        s = r.barTop;
      return a < s ? n - ( t - 21 ) : n + ( t - 21 );
    },
  };

  onScroll( r: any ) {
    if ( ! this.scroller ) return;
    const t = this.scroller.scrollTop;
    if ( t && t === this.currentScrollTop ) return;
    this.currentScrollTop = t;
    this.update();
    this.props.onScroll &&
    this.props.onScroll( r, {
      scrollTop: this.currentScrollTop,
    } );
  }

  onDragStart = (e: any) => {
    e.preventDefault();
    e.stopPropagation();
    const a = this.getScrollHeight();
    const n = this.getTrackHeight();
    this.dragging = true;
    this.delta.y = e.clientY;
    this.delta.top =
      'top' === this.props.anchor ? this.currentScrollTop : a - this.currentScrollTop;
    this.delta.speed = a / n;
    document.addEventListener( 'mousemove', this.onDrag );
    document.addEventListener( 'mouseup', this.onDragEnd );
    document.addEventListener( 'mouseleave', this.onDragEnd );
  };

  onDragEnd = () => {
    this.dragging = false;
    document.removeEventListener( 'mousemove', this.onDrag );
    document.removeEventListener( 'mouseup', this.onDragEnd );
    document.removeEventListener( 'mouseleave', this.onDragEnd );
  };

  onDrag = (e: any) => {
    e.preventDefault();
    e.stopPropagation();
    const a = e.clientY;
    const n = this.getScrollHeight();
    const t = ( a - (this.delta.y || 0) ) * (this.delta.speed || 1);
    const s = 'top' === this.props.anchor ? (this.delta.top || 0) + t : n - ( (this.delta.top || 0) - t );
    this.scrollTop( s );
  };

  onTrackDown = (a: any) => {
    console.log('track down');
    const n = this;
    a.preventDefault();
    a.stopPropagation();
    const t = this.props.onTrackClick || this.track.to;
    const s = a.nativeEvent.offsetY;
    const l = function e() {
      if ( s >= n.state.top && s <= n.state.top + n.state.height ) {
        clearInterval( n.trackInterval );
        return;
      }
      const r = t( {
        y: s,
        scrollTop: n.currentScrollTop,
        scrollHeight: n.getScrollHeight(),
        trackHeight: n.getTrackHeight(),
        barTop: n.state.top,
        barHeight: n.state.height,
      } );
      n.scrollWithAnimation( r );
    };
    l();
    clearInterval( this.trackInterval );
    clearTimeout( this.trackTimeout );
    this.trackTimeout = setTimeout( function() {
      l();
      n.trackInterval = setInterval( l, 200 );
    }, 500 );
    window.addEventListener( 'mouseup', this.onTrackUp );
  };

  onTrackUp = () => {
    this.trackAnimation && this.trackAnimation.cancel();
    clearInterval( this.trackInterval );
    clearTimeout( this.trackTimeout );
    window.removeEventListener( 'mouseup', this.onTrackUp );
  };

  setRef( scroller: any ) {
    this.scroller = scroller;
  }

  setTrackRef( track: any ) {
    this.track = track;
  }

  getScrollHeight = () => {
    const r = this.props.contentHeight;
    return 'undefined' !== typeof r ? r : this.scroller.scrollHeight;
  };

  getBarHeight(n?: any) {
    const r = this.getScrollHeight();
    const t = this.props.height;
    if ( 0 === r ) return 0;
    return ( t / r ) * t;
  }

  getTrackHeight = () => {
    return this.track && this.track.offsetHeight;
  };

  scrollTop( r: any ) {
    if ( ! this.scroller ) return null;
    if ( void 0 !== r ) {
      this.scroller.scrollTop = r;
      return this.scroller.scrollTop;
    }
    return this.currentScrollTop;
  }

  scrollWithAnimation = (r: any) => {
    // console.log("rr", r);
    // var t = this;
    // var n = 0;
    // var a = function e( rr ) {
    // 	var a = rr.value,
    // 		s = rr.nextTick;
    //
    // 	var l = t.scroller.scrollTop + a + n;
    // 	var i = Math.round( l );
    // 	n = l - i;
    // 	t.scrollTop( i );
    // 	// s();
    // };
    // this.trackAnimation && this.trackAnimation.cancel();
    // this.trackAnimation = Animate( {
    // 	fromValue: this.scroller.scrollTop,
    // 	toValue: r,
    // 	duration: 200,
    // 	delta: true,
    //   onUpdate: a,
    // } );
    // return this.trackAnimation;
  };

  update = () => {
    if ( ! this.scroller ) return;
    const r = this.props.height;
    const t = this.currentScrollTop;
    const n = this.getScrollHeight();
    let a = this.getBarHeight( n );
    a = a < 50 ? 50 : a;
    const s = ( t / ( n - r ) ) * ( this.getTrackHeight() - a );
    const l = n <= r;
    this.setState( {
      height: a,
      top: s,
      hidden: l,
    });
  };

  render() {
    const {
      width,
      height,
      isEmpty,
    } = this.props;
    // const rootClass = classNames( 'c_scrollbar', {
    //   'c_scrollbar__hidden': this.state.hidden,
    //   'c_scrollbar__monkey': this.props.monkey,
    //   'c_scrollbar__fade': this.props.fade,
    // }, this.props.className );

    const _rootClasses = classNames(
      gs.c_scrollbar,
      gs.position__relative,
      gs.overflow_y__hidden,
      gs.overflow_x__hidden,
      ss.scrollbar__fade,
      this.props.className,
    );

    return (
      <div
        className={ _rootClasses }
        style={ { width: width, height: height } }
        aria-label={ this.props.ariaLabel }
        onKeyDown={ this.props.onKeyDown }
      >
        <div
          role={ 'presentation' }
          className={classNames(
            gs.c_scrollbar__hider,
            gs.overflow_y__auto,
            gs.overflow_x__hidden,
            gs.outline__none,
            gs.position__absolute,
            gs.top__0,
            gs.left__0,
            gs.bottom__0,
            gs.right__50m,
            gs.width__100_percent,
            gs.height__100_percent,
            gs.transform__translateZ0,
            gs.pseudo_webkit_scrollbar__none,
          )}
          onScroll={ this.onScroll }
          ref={ this.setRef }
        >
          <Content width={ width } role={ 'presentation' } onScroll={ this.props.onContentScroll }>
            { this.props.children }
          </Content>
        </div>

        <div
          role={ 'presentation' }
          className={classNames(
            gs.c_scrollbar__track,
            gs.position__absolute,
            gs.right__2,
            gs.top__0,
            gs.bottom__0,
            gs.width__8,
            gs.opacity__1,
            gs.c_scrollbar__track
          )}
          onMouseDown={ this.onTrackDown }
          onMouseUp={ this.onTrackUp }
          ref={ this.setTrackRef }
        >
          <div
            role={ 'presentation' }
            className={classNames(
              gs.c_scrollbar__bar,
              gs.position__absolute,
              gs.top__0,
              gs.left__0,
              gs.width__100_percent,
              gs.border_radius__4,
              gs.opacity__0,
              gs.cursor__default,
              gs.outline__none,
              gs.z_index__1,
              gs.will_change__transform,
              ss.scrollbar__bar,
              gs.background__black_alpha_40, {
                [gs.opacity__0]: this.state.hidden,
              }
            )}
            style={ {
              height: this.state.height,
              transform: 'translateY(' + Math.floor( this.state.top ) + 'px)',
              background: this.props.color,
              opacity: isEmpty && 0,
            } }
            onMouseDown={ this.onDragStart }
            onClick={ E }
            tabIndex={ -1 }
          />
        </div>
      </div>
    );
  }
}
